# -*- encoding: utf-8 -*-

"""
A Short Module to Handle Input/Output for JSON Response

The response received from the external API sources are parsed and
returned as per user requirements.
"""

from forexrates.io import dataframe
